"use strict";
var program = require("commander");
var fs = require("fs");
var jsonPropertyFilter_1 = require("./jsonPropertyFilter");
program
    .version('1.0.0')
    .description('Filter a JSON file by including or excluding properties.')
    .usage('<file>')
    .option('-f, --filters <filters>', 'Add include and exclude filters.')
    .option('-o, --out <file>', 'Specifies the output file.')
    .option('-p, --pretty', 'Display results in an easy-to-read format.')
    .action(_apply)
    .parse(process.argv);
if (!program.args.length) {
    program.help();
}
function _apply(file, options) {
    fs.readFile(file, function (error, data) {
        _assertReadFile(file, error);
        var jsonObject = _transformToJsonObject(data);
        var jsonPropertyFilter = new jsonPropertyFilter_1.JsonPropertyFilter(options.filters);
        var filteredJsonObject = _filterJsonObject(jsonPropertyFilter, jsonObject);
        var formattedJsonObject = _format(filteredJsonObject, options.pretty);
        _out(formattedJsonObject, options.out);
    });
}
function _assertReadFile(file, error) {
    if (error) {
        throw new Error("An error occurred while the reading of file: " + file);
    }
}
function _assertWriteFile(error) {
    if (error) {
        throw new Error("An error occurred while saving of filtered JSON object.");
    }
}
function _filterJsonObject(jsonPropertyFilter, jsonObject) {
    return jsonPropertyFilter.apply(jsonObject);
}
function _format(filteredJsonObject, pretty) {
    if (pretty) {
        return JSON.stringify(filteredJsonObject, null, 2);
    }
    else {
        return JSON.stringify(filteredJsonObject);
    }
}
function _out(jsonObject, outputFile) {
    if (outputFile) {
        _writeResult(outputFile, jsonObject);
    }
    else {
        console.log(jsonObject);
    }
}
function _transformToJsonObject(data) {
    var fileContent = data.toString();
    try {
        return JSON.parse(fileContent);
    }
    catch (e) {
        throw new Error("An error occurred while processing of file content in a JSON object.");
    }
}
function _writeOutputFile(error) {
    _assertWriteFile(error);
}
function _writeResult(outputFile, data) {
    fs.writeFile(outputFile, data, _writeOutputFile);
}
